<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace ILIAS\Plugin\PlagScanExercise\Form;

use ilCheckboxInputGUI;
use ilDateTime;
use ilDateTimeInputGUI;
use ilExAssignment;
use ilExAssignmentEditorGUI;
use ilExAssignmentReminder;
use ilExAssignmentTeam;
use ilFileInputGUI;
use ilFileWizardInputGUI;
use ilFormSectionHeaderGUI;
use ILIAS\Plugin\PlagScanExercise\PlagScan\AssignmentConfigPersistence;
use ilNumberInputGUI;
use ilPropertyFormGUI;
use ilRadioGroupInputGUI;
use ilRadioOption;
use ilTextAreaInputGUI;
use ilTextInputGUI;
use ilUtil;

class EditorGUI extends ilExAssignmentEditorGUI
{
    public function getTextAssignmentCreationForm(): ilPropertyFormGUI
    {
        return $this->initAssignmentForm(5, 'create');
    }

    public function getTextAssignmentUpdateForm(): ilPropertyFormGUI
    {
        global $DIC;

        $form = $this->initAssignmentForm(5, 'edit');
        $this->getAssignmentValues($form);

        $persistence = new AssignmentConfigPersistence($DIC->database());
        $ass_id = (int) ($DIC->http()->request()->getQueryParams()['ass_id'] ?? '');
        if ($ass_id !== 0) {
            $settings = $persistence->getByAssignmentId($ass_id);
            $form->getItemByPostVar('psx_activation')->setChecked($settings->getActivation());
            $form->getItemByPostVar('psx_checkstart')->setValue($settings->getStartCheck());
            $form->getItemByPostVar('psx_resultshare')->setValue($settings->getShareSetting());
            $form->getItemByPostVar('psx_resulttype')->setValue($settings->getShareType());
        }

        return $form;
    }

    public function getUploadAssignmentCreationForm(): ilPropertyFormGUI
    {
        return $this->initAssignmentForm(1, 'create');
    }

    public function getUploadAssignmentUpdateForm(): ilPropertyFormGUI
    {
        global $DIC;

        $form = $this->initAssignmentForm(1, 'edit');
        $this->getAssignmentValues($form);

        $persistence = new AssignmentConfigPersistence($DIC->database());
        $ass_id = (int) ($DIC->http()->request()->getQueryParams()['ass_id'] ?? '');
        if ($ass_id !== 0) {
            $settings = $persistence->getByAssignmentId($ass_id);
            $form->getItemByPostVar('psx_activation')->setChecked($settings->getActivation());
            $form->getItemByPostVar('psx_checkstart')->setValue($settings->getStartCheck());
            $form->getItemByPostVar('psx_resultshare')->setValue($settings->getShareSetting());
            $form->getItemByPostVar('psx_resulttype')->setValue($settings->getShareType());
        }

        return $form;
    }

    protected function initAssignmentForm(int $a_type, string $a_mode = 'create'): ilPropertyFormGUI
    {
        $ass_type = $this->types->getById($a_type);
        $ass_type_gui = $this->type_guis->getById($a_type);
        $this->ctrl->setParameterByClass(ilExAssignmentEditorGUI::class, 'ass_type', $a_type);

        $this->lng->loadLanguageModule('form');
        $form = new ilPropertyFormGUI();
        if ($a_mode === 'edit') {
            $form->setTitle($this->lng->txt('exc_edit_assignment'));
        } else {
            $form->setTitle($this->lng->txt('exc_new_assignment'));
        }
        $form->setFormAction($this->ctrl->getFormActionByClass(ilExAssignmentEditorGUI::class));

        $title= new ilTextInputGUI($this->lng->txt('title'), 'title');
        $title->setMaxLength(200);
        $title->setRequired(true);
        $form->addItem($title);

        $ty = $this->getTypeDropdown();
        $ty->setValue($a_type);
        $ty->setDisabled(true);
        $form->addItem($ty);

        $ass_type_gui->addEditFormCustomProperties($form);

        if ($ass_type->usesTeams()) {
            if ($a_mode === 'edit') {
                $has_teams = ilExAssignmentTeam::getAssignmentTeamMap($this->assignment->getId()) !== [];
            } else {
                $has_teams = false;
            }

            $rd_team = new ilRadioGroupInputGUI($this->lng->txt('exc_team_formation'), 'team_creator');

            $radio_participants = new ilRadioOption(
                $this->lng->txt('exc_team_by_participants'),
                (string) ilExAssignment::TEAMS_FORMED_BY_PARTICIPANTS,
                $this->lng->txt('exc_team_by_participants_info')
            );

            $radio_tutors = new ilRadioOption(
                $this->lng->txt('exc_team_by_tutors'),
                (string) ilExAssignment::TEAMS_FORMED_BY_TUTOR,
                $this->lng->txt('exc_team_by_tutors_info')
            );

            if (!$has_teams) {
                $rd_creation_method = new ilRadioGroupInputGUI($this->lng->txt('exc_team_creation'), 'team_creation');
                $rd_creation_method->setRequired(true);

                $rd_creation_manual = new ilRadioOption(
                    $this->lng->txt('exc_team_by_tutors_manual'),
                    '0',
                    $this->lng->txt('exc_team_by_tutors_manual_info')
                );
                $rd_creation_method->addOption($rd_creation_manual);

                $rd_creation_random = new ilRadioOption(
                    $this->lng->txt('exc_team_by_random'),
                    (string) ilExAssignment::TEAMS_FORMED_BY_RANDOM,
                    $this->lng->txt('exc_team_by_random_info') . '<br>' . $this->lng->txt('exc_total_members') . ': ' . $this->getExerciseTotalMembers()
                );
                $rd_creation_method->addOption($rd_creation_random);

                $number_teams = new ilNumberInputGUI($this->lng->txt('exc_num_teams'), 'number_teams');
                $number_teams->setSize(3);
                $number_teams->setMinValue(1);
                $number_teams->setMaxValue($this->getExerciseTotalMembers());
                $number_teams->setRequired(true);
                $number_teams->setSuffix($this->lng->txt('exc_team_assignment_adopt_teams'));
                $rd_creation_random->addSubItem($number_teams);

                $min_team_participants = new ilNumberInputGUI($this->lng->txt('exc_min_team_participants'), 'min_participants_team');
                $min_team_participants->setSize(3);
                $min_team_participants->setMinValue(1);
                $min_team_participants->setMaxValue($this->getExerciseTotalMembers());
                $min_team_participants->setRequired(true);
                $min_team_participants->setSuffix($this->lng->txt('exc_participants'));
                $rd_creation_random->addSubItem($min_team_participants);

                $max_team_participants = new ilNumberInputGUI($this->lng->txt('exc_max_team_participants'), 'max_participants_team');
                $max_team_participants->setSize(3);
                $max_team_participants->setMinValue(1);
                $max_team_participants->setMaxValue($this->getExerciseTotalMembers());
                $max_team_participants->setRequired(true);
                $max_team_participants->setSuffix($this->lng->txt('exc_participants'));
                $rd_creation_random->addSubItem($max_team_participants);

                $options = ilExAssignmentTeam::getAdoptableTeamAssignments($this->exercise_id);
                if (count($options)) {
                    $radio_assignment = new ilRadioOption(
                        $this->lng->txt('exc_team_by_assignment'),
                        (string) ilExAssignment::TEAMS_FORMED_BY_ASSIGNMENT,
                        $this->lng->txt('exc_team_by_assignment_info')
                    );

                    $radio_assignment_adopt = new ilRadioGroupInputGUI($this->lng->txt('exc_assignment'), 'ass_adpt');
                    $radio_assignment_adopt->setRequired(true);
                    $radio_assignment_adopt->addOption(new ilRadioOption($this->lng->txt('exc_team_assignment_adopt_none'), '-1'));

                    foreach ($options as $id => $item) {
                        $option = new ilRadioOption($item['title'], $id);
                        $option->setInfo($this->lng->txt('exc_team_assignment_adopt_teams') . ': ' . $item['teams']);
                        $radio_assignment_adopt->addOption($option);
                    }
                    $radio_assignment->addSubItem($radio_assignment_adopt);
                    $rd_creation_method->addOption($radio_assignment);
                }

                $radio_tutors->addSubItem($rd_creation_method);
            }
            $rd_team->addOption($radio_participants);
            $rd_team->addOption($radio_tutors);
            $form->addItem($rd_team);

            if ($has_teams) {
                $rd_team->setDisabled(true);
            }
        }

        $cb = new ilCheckboxInputGUI($this->lng->txt('exc_mandatory'), 'mandatory');
        $cb->setInfo($this->lng->txt('exc_mandatory_info'));
        $cb->setChecked(true);
        $form->addItem($cb);

        $sub_header = new ilFormSectionHeaderGUI();
        $sub_header->setTitle($this->lng->txt('exc_work_instructions'));
        $form->addItem($sub_header);

        $desc_input = new ilTextAreaInputGUI($this->lng->txt('exc_instruction'), 'instruction');
        $desc_input->setRows(20);
        $desc_input->setUseRte(true);
        $desc_input->setRteTagSet('mini');
        $form->addItem($desc_input);

        if ($a_mode === 'create') {
            $files = new ilFileWizardInputGUI($this->lng->txt('objs_file'), 'files');
            $files->setFilenames(['']);
            $form->addItem($files);
        }

        $sub_header = new ilFormSectionHeaderGUI();
        $sub_header->setTitle($this->lng->txt('exc_schedule'));
        $form->addItem($sub_header);

        $start_date = new ilDateTimeInputGUI($this->lng->txt('exc_start_time'), 'start_time');
        $start_date->setShowTime(true);
        $form->addItem($start_date);

        $radg = new ilRadioGroupInputGUI($this->lng->txt('exc_deadline'), 'deadline_mode');
        $radg->setValue('0');
        $op1 = new ilRadioOption($this->lng->txt('exc_fixed_date'), '0', $this->lng->txt('exc_fixed_date_info'));
        $radg->addOption($op1);
        $op2 = new ilRadioOption($this->lng->txt('exc_relative_date'), '1', $this->lng->txt('exc_relative_date_info'));
        $radg->addOption($op2);
        $form->addItem($radg);

        $deadline = new ilDateTimeInputGUI($this->lng->txt('date'), 'deadline');
        $deadline->setShowTime(true);
        $op1->addSubItem($deadline);

        $deadline2 = new ilDateTimeInputGUI($this->lng->txt('exc_deadline_extended'), 'deadline2');
        $deadline2->setInfo($this->lng->txt('exc_deadline_extended_info'));
        $deadline2->setShowTime(true);
        $deadline->addSubItem($deadline2);

        $rmd_submit = new ilCheckboxInputGUI($this->lng->txt('exc_reminder_submit_setting'), 'rmd_submit_status');

        $rmd_submit_start = new ilNumberInputGUI($this->lng->txt('exc_reminder_start'), 'rmd_submit_start');
        $rmd_submit_start->setSize(3);
        $rmd_submit_start->setMaxLength(3);
        $rmd_submit_start->setSuffix($this->lng->txt('days'));
        $rmd_submit_start->setInfo($this->lng->txt('exc_reminder_start_info'));
        $rmd_submit_start->setRequired(true);
        $rmd_submit_start->setMinValue(1);
        $rmd_submit->addSubItem($rmd_submit_start);

        $rmd_submit_frequency = new ilNumberInputGUI($this->lng->txt('exc_reminder_frequency'), 'rmd_submit_freq');
        $rmd_submit_frequency->setSize(3);
        $rmd_submit_frequency->setMaxLength(3);
        $rmd_submit_frequency->setSuffix($this->lng->txt('days'));
        $rmd_submit_frequency->setRequired(true);
        $rmd_submit_frequency->setMinValue(1);
        $rmd_submit->addSubItem($rmd_submit_frequency);

        $rmd_submit_end = new ilDateTimeInputGUI($this->lng->txt('exc_reminder_end'), 'rmd_submit_end');
        $rmd_submit_end->setRequired(true);
        $rmd_submit->addSubItem($rmd_submit_end);

        $rmd_submit->addSubItem($this->addMailTemplatesRadio(ilExAssignmentReminder::SUBMIT_REMINDER));

        $rmd_grade = new ilCheckboxInputGUI($this->lng->txt('exc_reminder_grade_setting'), 'rmd_grade_status');

        $rmd_grade_frequency = new ilNumberInputGUI($this->lng->txt('exc_reminder_frequency'), 'rmd_grade_freq');
        $rmd_grade_frequency->setSize(3);
        $rmd_grade_frequency->setMaxLength(3);
        $rmd_grade_frequency->setSuffix($this->lng->txt('days'));
        $rmd_grade_frequency->setRequired(true);
        $rmd_grade_frequency->setMinValue(1);
        $rmd_grade->addSubItem($rmd_grade_frequency);

        $rmd_grade_end = new ilDateTimeInputGUI($this->lng->txt('exc_reminder_end'), 'rmd_grade_end');
        $rmd_grade_end->setRequired(true);
        $rmd_grade->addSubItem($rmd_grade_end);

        $rmd_grade->addSubItem($this->addMailTemplatesRadio(ilExAssignmentReminder::GRADE_REMINDER));

        $form->addItem($rmd_submit);
        $form->addItem($rmd_grade);

        $ti = new ilNumberInputGUI($this->lng->txt('days'), 'relative_deadline');
        $ti->setMaxLength(3);
        $ti->setSize(3);
        $op2->addSubItem($ti);

        if ($ass_type->usesFileUpload()) {
            $sub_header = new ilFormSectionHeaderGUI();
            $sub_header->setTitle($ass_type->getTitle());
            $form->addItem($sub_header);

            $max_file_tgl = new ilCheckboxInputGUI($this->lng->txt('exc_max_file_tgl'), 'max_file_tgl');
            $form->addItem($max_file_tgl);

            $max_file = new ilNumberInputGUI($this->lng->txt('exc_max_file'), 'max_file');
            $max_file->setInfo($this->lng->txt('exc_max_file_info'));
            $max_file->setRequired(true);
            $max_file->setSize(3);
            $max_file->setMinValue(1);
            $max_file_tgl->addSubItem($max_file);
        }

        $sub_header = new ilFormSectionHeaderGUI();
        $sub_header->setTitle($this->lng->txt('exc_after_submission'));
        $form->addItem($sub_header);
        if (!$ass_type->usesTeams()) {
            $peer = new ilCheckboxInputGUI($this->lng->txt('exc_peer_review'), 'peer');
            $peer->setInfo($this->lng->txt('exc_peer_review_ass_setting_info'));
            $form->addItem($peer);
        }

        $fb = new ilCheckboxInputGUI($this->lng->txt('exc_global_feedback_file'), 'fb');
        $form->addItem($fb);

        $fb_file = new ilFileInputGUI($this->lng->txt('file'), 'fb_file');
        $fb_file->setRequired(true);
        $fb->addSubItem($fb_file);

        $fb_date = new ilRadioGroupInputGUI($this->lng->txt('exc_global_feedback_file_date'), 'fb_date');
        $fb_date->setRequired(true);
        $fb_date->addOption(new ilRadioOption($this->lng->txt('exc_global_feedback_file_date_deadline'), (string) ilExAssignment::FEEDBACK_DATE_DEADLINE));
        $fb_date->addOption(new ilRadioOption($this->lng->txt('exc_global_feedback_file_date_upload'), (string) ilExAssignment::FEEDBACK_DATE_SUBMISSION));

        $fb_date_custom_date = new ilDateTimeInputGUI($this->lng->txt('date'), 'fb_date_custom');
        $fb_date_custom_date->setRequired(true);
        $fb_date_custom_date->setShowTime(true);
        $fb_date_custom_option = new ilRadioOption($this->lng->txt('exc_global_feedback_file_after_date'), (string) ilExAssignment::FEEDBACK_DATE_CUSTOM);
        $fb_date_custom_option->addSubItem($fb_date_custom_date);
        $fb_date->addOption($fb_date_custom_option);

        $fb->addSubItem($fb_date);

        $fb_cron = new ilCheckboxInputGUI($this->lng->txt('exc_global_feedback_file_cron'), 'fb_cron');
        $fb_cron->setInfo($this->lng->txt('exc_global_feedback_file_cron_info'));
        $fb->addSubItem($fb_cron);

        $plag_scan_additions = new FormSection();
        $plag_scan_additions->addPlagScanConfig($form);

        if ($a_mode === 'create') {
            $form->addCommandButton('saveAssignment', $this->lng->txt('save'));
        } else {
            $form->addCommandButton('updateAssignment', $this->lng->txt('save'));
        }
        $form->addCommandButton('listAssignments', $this->lng->txt('cancel'));

        return $form;
    }

    public function getAssignmentValues(ilPropertyFormGUI $a_form): void
    {
        $ass_type_gui = $this->type_guis->getById($this->assignment->getType());

        $values = [];
        $values["type"] = $this->assignment->getType();
        $values["title"] = $this->assignment->getTitle();
        $values["mandatory"] = $this->assignment->getMandatory();
        $values["instruction"] = $this->assignment->getInstruction();

        if ($this->assignment->getStartTime()) {
            $values["start_time"] = new ilDateTime($this->assignment->getStartTime(), IL_CAL_UNIX);
        }

        if ($this->assignment->getAssignmentType()->usesFileUpload()) {
            if ($this->assignment->getMaxFile()) {
                $values["max_file_tgl"] = true;
                $values["max_file"] = $this->assignment->getMaxFile();
            }
        }

        if ($this->assignment->getAssignmentType()->usesTeams()) {
            $values["team_creator"] = $this->assignment->getTeamTutor();
        }

        if ($this->assignment->getFeedbackDateCustom()) {
            $values["fb_date_custom"] = new ilDateTime($this->assignment->getFeedbackDateCustom(), IL_CAL_UNIX);
        }

        $rmd_sub = new ilExAssignmentReminder($this->exercise_id, $this->assignment->getId(), ilExAssignmentReminder::SUBMIT_REMINDER);
        if ($rmd_sub->getReminderStatus()) {
            $values["rmd_submit_status"] = $rmd_sub->getReminderStatus();
            $values["rmd_submit_start"] = $rmd_sub->getReminderStart();
            $values["rmd_submit_end"] = new ilDateTime($rmd_sub->getReminderEnd(), IL_CAL_UNIX);
            $values["rmd_submit_freq"] = $rmd_sub->getReminderFrequency();
            $values["rmd_submit_template_id"] = $rmd_sub->getReminderMailTemplate();
        }

        $rmd_grade = new ilExAssignmentReminder($this->exercise_id, $this->assignment->getId(), ilExAssignmentReminder::GRADE_REMINDER);
        if ($rmd_grade->getReminderStatus()) {
            $values["rmd_grade_status"] = $rmd_grade->getReminderStatus();
            $values["rmd_grade_end"] = new ilDateTime($rmd_grade->getReminderEnd(), IL_CAL_UNIX);
            $values["rmd_grade_freq"] = $rmd_grade->getReminderFrequency();
            $values["rmd_grade_template_id"] = $rmd_grade->getReminderMailTemplate();
        }

        $type_values = $ass_type_gui->getFormValuesArray($this->assignment);
        $values = array_merge($values, $type_values);

        $values["deadline_mode"] = $this->assignment->getDeadlineMode();
        $values["relative_deadline"] = $this->assignment->getRelativeDeadline();
        $values["rel_deadline_last_subm"] = new ilDateTime($this->assignment->getRelDeadlineLastSubmission(), IL_CAL_UNIX);

        $a_form->setValuesByArray($values);

        if ($this->assignment->getFeedbackFile()) {
            $a_form->getItemByPostVar("fb")->setChecked(true);
            $a_form->getItemByPostVar("fb_file")->setValue(basename($this->assignment->getGlobalFeedbackFilePath()));
            $a_form->getItemByPostVar("fb_file")->setRequired(false);
            $a_form->getItemByPostVar("fb_file")->setInfo(
                '<a href="' . $this->ctrl->getLinkTargetByClass('ilExAssignmentEditorGUI', "downloadGlobalFeedbackFile") . '">' .
                $this->lng->txt("download") . '</a>'
            );
        }
        $a_form->getItemByPostVar("fb_cron")->setChecked($this->assignment->hasFeedbackCron());
        $a_form->getItemByPostVar("fb_date")->setValue((string) $this->assignment->getFeedbackDate());

        $this->handleDisabledFields($a_form, true);
    }

}
