<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace ILIAS\Plugin\PlagScanExercise\PlagScan;

use ilDBConstants;
use ilDBInterface;
use ilExAssignment;
use ilObject;
use ilUtil;

class AssignmentConfigPersistence
{
    protected ilDBInterface $db;
    protected int $id;

    public function __construct(ilDBInterface $db)
    {
        global $DIC;

        $this->db = $db;
        $this->id = ilObject::_lookupObjectId((int) ($DIC->http()->request()->getQueryParams()['ref_id'] ?? ''));
    }

    public function store(AssignmentConfigData $data): void
    {
        $query = 'DELETE FROM psx_assignments WHERE assignment_exc_obj = '
            . $this->db->quote($data->getExerciseObjId(), ilDBConstants::T_INTEGER)
            . ' AND assignment_id = ' . $this->db->quote($data->getAssignmentId(), ilDBConstants::T_INTEGER);
        $this->db->query($query);
        $this->db->insert(
            'psx_assignments',
            [
                'id' => [ilDBConstants::T_INTEGER, $this->db->nextId('psx_assignments')],
                'assignment_exc_obj' => [ilDBConstants::T_INTEGER, $data->getExerciseObjId()],
                'assignment_id' => [ilDBConstants::T_INTEGER, $data->getAssignmentId()],
                'activation' => [ilDBConstants::T_INTEGER, $data->getActivation()],
                'start' => [ilDBConstants::T_TEXT, $data->getStartCheck()],
                'share_result' => [ilDBConstants::T_TEXT, $data->getShareSetting()],
                'result_type' => [ilDBConstants::T_TEXT, $data->getShareType()]
            ]
        );
    }

    public function getByAssignmentId(int $ass_id): AssignmentConfigData
    {
        $result = $this->db->query('SELECT * FROM psx_assignments WHERE assignment_id = '
           . $this->db->quote($ass_id, ilDBConstants::T_INTEGER));
        $row = $this->db->fetchAssoc($result);
        if ($row !== null) {
            $data = new AssignmentConfigData(
                (int) $row['assignment_exc_obj'],
                (int) $row['assignment_id'],
                $row['activation'] === '1',
                $row['start'],
                $row['share_result'],
                $row['result_type']
            );
        } else {
            $data = new AssignmentConfigData(
                $this->id,
                $ass_id,
                false,
                'manual',
                'none',
                'none'
            );
        }
        return $data;
    }

    public function getByAssignmentOrderInRef(int $ref_id, int $number): AssignmentConfigData
    {
        $ass_data = ilExAssignment::getInstancesByExercise(ilObject::_lookupObjectId($ref_id));
        $result = $this->db->query('SELECT * FROM psx_assignments WHERE assignment_id = '
           . $this->db->quote($ass_data[$number]->getId(), ilDBConstants::T_INTEGER));
        $row = $this->db->fetchAssoc($result);
        if ($row !== null) {
            $data = new AssignmentConfigData(
                (int) $row['assignment_exc_obj'],
                (int) $row['assignment_id'],
                $row['activation'] === '1',
                $row['start'],
                $row['share_result'],
                $row['result_type']
            );
        } else {
            $data = new AssignmentConfigData(
                $this->id,
                $ass_data[$number]->getId(),
                false,
                'manual',
                'none',
                'none'
            );
        }
        return $data;
    }
}
