<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace PlagScan\Helper;

class Documents extends Base
{
    /**
     * @return array|string
     */
    public function get(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id, 'GET', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function getStatus(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id . '/status', 'GET', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function add(string $file_data, bool $plain, int $user_id = 0)
    {
        if ($plain) {
            $data['textdata'] = $file_data;
        } else {
            $data['fileUpload'] = $file_data;
        }
        if ($user_id) {
            $data['userID'] = $user_id;
        }
        return $this->handler->request('documents', 'POST', $this->access_token, $data);
    }

    /**
     * @return array|string
     */
    public function delete(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id, 'DELETE', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function archive(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id . '/archive', 'PUT', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function check(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id . '/check', 'PUT', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function retrieveReport(int $doc_id, int $mode = 0, int $user_id = 0)
    {
        $data['mode'] = $mode;
        if ($user_id) {
            $data['userID'] = $user_id;
        }
        return $this->handler->request('documents/' . $doc_id . '/retrieve', 'GET', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function retrieveAllSources(int $doc_id, string $format = 'old_html')
    {
        $data['format'] = $format;
        return $this->handler->request('documents/' . $doc_id . '/sources', 'GET', $this->access_token, $data);
    }

    /**
     * @return array|string
     */
    public function retrieveHighlightedSources(int $doc_id, string $source, string $format = 'old_html')
    {
        $data['source'] = $source;
        $data['format'] = $format;
        return $this->handler->request('documents/' . $doc_id . '/highlight', 'GET', $this->access_token, $data);
    }

    /**
     * @return array|string
     */
    public function privatize(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id . '/private', 'PUT', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function publish(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id . '/private', 'DELETE', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function share(int $doc_id, array $config = [])
    {
        return $this->handler->request('documents/' . $doc_id . '/share', 'PUT', $this->access_token, $config);
    }

    /**
     * @return array|string
     */
    public function unshare(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id . '/share', 'DELETE', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function getShareLink(int $doc_id)
    {
        return $this->handler->request('documents/' . $doc_id . '/share', 'GET', $this->access_token);
    }
}
