<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace PlagScan\Helper;

class Submissions extends Base
{
    /**
     * @return array|string
     */
    public function getAll(int $user_id = 0)
    {
        $data = [];
        if ($user_id) {
            $data['ownerID'] = $user_id;
        }

        return $this->handler->request('submissions', 'GET', $this->access_token, $data);
    }

    /**
     * @return array|string
     */
    public function add(
        string $title,
        int $max_uploads,
        int $share,
        int $check_deadline,
        int $notify_deadline,
        int $enable_resubmission,
        int $check_on_upload,
        int $type,
        array $configuration = []
    ) {
        $data['title'] = $title;
        $data['maxUploads'] = $max_uploads;
        $data['share'] = $share;
        $data['checkDeadline'] = $check_deadline;
        $data['notifyDeadline'] = $notify_deadline;
        $data['enableResubmission'] = $enable_resubmission;
        $data['checkOnUpload'] = $check_on_upload;
        $data['type'] = $type;

        $data += $configuration;

        return $this->handler->request('submissions', 'POST', $this->access_token, $data);
    }

    /**
     * @return array|string
     */
    public function change(int $submission_id, array $changes)
    {
        return $this->handler->request('submissions/' . $submission_id, 'PATCH', $this->access_token, $changes);
    }

    /**
     * @return array|string
     */
    public function delete(int $submission_id)
    {
        return $this->handler->request('submissions/' . $submission_id, 'DELETE', $this->access_token);
    }

    /**
     * @return array|string
     */
    public function getDocuments(int $submission_id, int $user_id)
    {
        $data['ownerID'] = $user_id;
        return $this->handler->request('submissions/' . $submission_id . '/documents', 'GET', $this->access_token, $data);
    }

    /**
     * @return array|string
     */
    public function addDocument(
        int $submission_id,
        string $file_upload,
        string $title,
        string $firstname,
        string $lastname,
        string $email,
        int $send_results = 0,
        int $save_origin = 0
    ) {
        $data['fileUpload'] = $file_upload;
        $data['title'] = $title;
        $data['firstname'] = $firstname;
        $data['lastname'] = $lastname;
        $data['email'] = $email;
        $data['sendResults'] = $send_results;
        $data['saveOrig'] = $save_origin;
        return $this->handler->request('submissions/' . $submission_id, 'POST', $this->access_token, $data);
    }

    /**
     * @return array|string
     */
    public function replaceDocument(int $submission_id, int $doc_id, string $file_upload, int $save_origin = 0)
    {
        $data['docID'] = $doc_id;
        $data['fileUpload'] = $file_upload;
        $data['saveOrig'] = $save_origin;
        return $this->handler->request('submissions/' . $submission_id, 'POST', $this->access_token, $data);
    }
}
